﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Controls.Primitives;
using System.ComponentModel;
using System.Windows.Threading;
using System.Threading;
using tx_spellwpf_wordmole.Properties;

namespace tx_spellwpf_wordmole
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private List<ToggleButton> tbCheckedButtons;
        private TXTextControl.WPF.Proofing.TXSpellChecker txSpellChecker1;
        private int iPoints;
        private DispatcherTimer dispatcherTimer;
        static Random _random = new Random();

        public MainWindow()
        {
            InitializeComponent();

            // create a new spell checker
            txSpellChecker1 = new TXTextControl.WPF.Proofing.TXSpellChecker();
            txSpellChecker1.Language = "(Auto)";
            txSpellChecker1.Create();

            InitializeBoard();
            RestartGame();
        }

        // initializes the board - create some toggle buttons for the characters
        private void InitializeBoard()
        {
            tbCheckedButtons = new List<ToggleButton>();

            for (int i = 0; i < 56; i++)
            {
                ToggleButton wordButton = new ToggleButton();
                wordButton.Width = 30;
                wordButton.Height = 30;
                wordButton.Content = GetLetter();

                wordButton.Click += new RoutedEventHandler(wordButton_Click);

                wordpanel.Children.Add(wordButton);
            }
        }

        private void RestartGame()
        {
            // restart the game, clear the button list
            tbCheckedButtons.Clear();
            lblPoints.Content = "";
            lblWord.Content = "";
            ClearCharacters();
            btnAccept.IsEnabled = false;
            iPoints = 0;
            
            lblHighScore.Content = "Highscore: " + Settings.Default.highscore.ToString();
            int count = 500;

            // set the timer as a delegate to be able to update the UI
            dispatcherTimer = new DispatcherTimer();

            dispatcherTimer.Interval = TimeSpan.FromMilliseconds(100);

            dispatcherTimer.Tick += delegate(object sender, EventArgs e)
            {
                lblTime.Content = "Time: " + count.ToString();
                count = (count - 1);

                if (count == -1)
                {
                    dispatcherTimer.Stop();

                    wordpanel.IsEnabled = false;
                    gameover.Visibility = System.Windows.Visibility.Visible;
                    Settings.Default.highscore = iPoints;
                    Settings.Default.Save();
                }
            };

            wordpanel.IsEnabled = true;
            gameover.Visibility = System.Windows.Visibility.Hidden;
            dispatcherTimer.Start();
        }

        void wordButton_Click(object sender, RoutedEventArgs e)
        {
            // a new character has been selected
            // button will be added to the list of buttons
            // if a button is toggled, remove all following buttons from the list
            ToggleButton btn = e.Source as ToggleButton;

            if (btn.IsChecked == true)
                tbCheckedButtons.Add(btn);
            else
            {
                int index = tbCheckedButtons.IndexOf(btn);
                tbCheckedButtons.RemoveRange(index, tbCheckedButtons.Count - index);

                ClearCharacters();
            }

            lblWord.Content = GetStringFromList(tbCheckedButtons);

            if (lblWord.Content.ToString().Length > 1)
                btnAccept.IsEnabled = true;
            else
                btnAccept.IsEnabled = false;
        }

        // clear all buttons and "uncheck" them
        private void ClearCharacters()
        {
            foreach (UIElement button in wordpanel.Children)
            {
                if (button is ToggleButton)
                {
                    ToggleButton tgbtn = button as ToggleButton;

                    if (tbCheckedButtons.Contains(tgbtn) == false)
                    {
                        tgbtn.IsChecked = false;
                    }
                }
            }
        }

        // returns the string from all stored buttons
        private string GetStringFromList(List<ToggleButton> List)
        {
            string word = String.Empty;

            foreach (ToggleButton btn in List)
            {
                word += btn.Content;
            }

            return word;
        }

        public char GetLetter()
        {
            int num = _random.Next(0, 26);
            char let = (char)('a' + num);
            return let;
        }

        private void btnAccept_Click(object sender, RoutedEventArgs e)
        {
            // check the currently created word
            txSpellChecker1.Check(lblWord.Content.ToString());

            // if word is correct, uncheck all buttons, count the points
            // and clear the button list
            if (txSpellChecker1.IncorrectWords.Count == 0)
            {
                foreach (ToggleButton btn in tbCheckedButtons)
                {
                    btn.Content = GetLetter();
                    iPoints++;
                }

                lblPoints.Content = iPoints;
                tbCheckedButtons.Clear();
                lblWord.Content = "";
                ClearCharacters();
                lblWord.Foreground = Brushes.White;
            }
            else
            {
                lblWord.Foreground = Brushes.Red;
            }

        }

        private void btnRestart_Click(object sender, RoutedEventArgs e)
        {
            RestartGame();
        }
    }
}
